Package: colordistance
Title: Distance Metrics for Image Color Similarity
Date: 2021-03-19
Version: 1.1.2
Authors@R: person("Hannah", "Weller", email = "hannahiweller@gmail.com", role = c("aut", "cre"))
Description: Loads and displays images, selectively masks specified background
    colors, bins pixels by color using either data-dependent or
    automatically generated color bins, quantitatively measures color
    similarity among images using one of several distance metrics for
    comparing pixel color clusters, and clusters images by object color
    similarity. Uses CIELAB, RGB, or HSV color spaces. Originally written
    for use with organism coloration (reef fish color diversity, butterfly
    mimicry, etc), but easily applicable for any image set.
Imports: jpeg, png, stats, clue, ape, mgcv, emdist, scatterplot3d,
        plotly, gplots, abind, magrittr, scales, qpdf, spatstat.geom
Depends: R (>= 3.4.0)
License: GPL-3
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.1.1
Suggests: knitr, rmarkdown, testthat
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2021-03-19 16:34:14 UTC; hannah
Author: Hannah Weller [aut, cre]
Maintainer: Hannah Weller <hannahiweller@gmail.com>
Repository: CRAN
Date/Publication: 2021-03-20 20:40:23 UTC
Built: R 4.6.0; ; 2026-04-30 09:14:34 UTC; unix
