# Generated by roxygen2: do not edit by hand

export(combineClusters)
export(combineList)
export(convertColorSpace)
export(exportTree)
export(extractClusters)
export(getColorDistanceMatrix)
export(getHistList)
export(getImageHist)
export(getImagePaths)
export(getKMeanColors)
export(getKMeansList)
export(getLabHist)
export(getLabHistList)
export(heatmapColorDistance)
export(imageClusterPipeline)
export(loadImage)
export(plotClusters)
export(plotClustersMulti)
export(plotHist)
export(plotImage)
export(plotPixels)
export(removeBackground)
export(scatter3dclusters)
import(grDevices)
import(graphics)
import(stats)
import(utils)
